from collections import defaultdict
from mock import MagicMock as Mock, patch
from pretend import stub

from helga.plugins import help


@patch('helga.plugins.help.registry')
def test_help_should_whisper(plugins):
    client = Mock()
    plugins.enabled_plugins = defaultdict(list)
    plugins.plugins = {}

    help.help(client, '#bots', 'me', 'foo', 'bar', [])

    assert client.me.called
    client.me.assertCalledWith('#bots', 'whispers to me')


@patch('helga.plugins.help.registry')
def test_help_gets_object_help_attrs(plugins):
    foo_plugin = stub(help='foo plugin')
    bar_plugin = stub()

    client = Mock()
    plugins.enabled_plugins = {'#bots': ['foo', 'bar']}
    plugins.plugins = {'foo': [foo_plugin], 'bar': [bar_plugin]}

    help.help(client, '#bots', 'me', 'help', 'help', [])
    output = "me, here are the plugins I know about\nfoo\nfoo plugin"

    client.msg.assertCalledWith('me', output)


@patch('helga.plugins.help.registry')
def test_help_gets_decorated_help_attrs(plugins):
    foo_plugin = stub(help='foo plugin')
    foo_fn = stub(_plugins=[foo_plugin])

    client = Mock()
    plugins.enabled_plugins = {'#bots': ['foo']}
    plugins.plugins = {'foo': [foo_fn]}

    help.help(client, '#bots', 'me', 'help', 'help', [])
    output = "me, here are the plugins I know about\nfoo\nfoo plugin"

    client.msg.assertCalledWith('me', output)


@patch('helga.plugins.help.registry')
def test_help_gets_multi_decorated_help_attrs(plugins):
    foo_plugin = stub(help='foo plugin')
    bar_plugin = stub(help='bar plugin')
    foo_fn = stub(_plugins=[foo_plugin, bar_plugin])

    client = Mock()
    plugins.enabled_plugins = {'#bots': ['foo']}
    plugins.plugins = {'foo': [foo_fn]}

    help.help(client, '#bots', 'me', 'help', 'help', [])
    output = "me, here are the plugins I know about\nfoo\nfoo plugin\nbar plugin"

    client.msg.assertCalledWith('me', output)


@patch('helga.plugins.help.registry')
def test_help_gets_single_plugin(plugins):
    foo_plugin = stub(help='foo plugin')
    bar_plugin = stub(help='bar plugin')

    client = Mock()
    plugins.enabled_plugins = {'#bots': ['foo', 'bar']}
    plugins.plugins = {'foo': [foo_plugin], 'bar': [bar_plugin]}

    help.help(client, '#bots', 'me', 'help', 'help', ['bar'])
    output = "me, here are the plugins I know about\nbar\nbar plugin"

    client.msg.assertCalledWith('me', output)


@patch('helga.plugins.help.registry')
def test_help_single_returns_unknown(plugins):
    foo_plugin = stub(help='foo plugin')
    bar_plugin = stub(help='bar plugin')

    client = Mock()
    plugins.enabled_plugins = {'#bots': ['foo', 'bar']}
    plugins.plugins = {'foo': [foo_plugin], 'bar': [bar_plugin]}

    resp = help.help(client, '#bots', 'me', 'help', 'help', ['baz'])

    assert resp == "Sorry me, I don't know about that plugin"
    assert not client.msg.called



from mock import patch
from pretend import stub

from helga.plugins import stfu


def test_stfu_preprocess_does_nothing():
    stfu.silenced = set()
    client = stub(nickname='helga')
    chan, nick, msg = stfu.stfu(client, '#bots', 'me', 'foo')
    assert msg == 'foo'


def test_stfu_preprocess_blanks_message_when_silenced():
    stfu.silenced = set(['#bots'])
    client = stub(nickname='helga')
    chan, nick, msg = stfu.stfu(client, '#bots', 'me', 'foo')
    assert msg == ''


def test_stfu_snark_on_private_message():
    stfu.silenced = set()
    resp = stfu.stfu(stub(nickname='helga'), 'me', 'me', 'helga stfu', 'stfu', [])
    assert resp in map(lambda x: x.format(nick='me'), stfu.snarks)


def test_stfu_silences_channel():
    stfu.silenced = set()
    resp = stfu.stfu(stub(nickname='helga'), '#bots', 'me', 'helga stfu', 'stfu', [])
    assert resp in map(lambda x: x.format(nick='me'), stfu.silence_acks)
    assert '#bots' in stfu.silenced


@patch('helga.plugins.stfu.reactor')
def test_stfu_for_some_time(reactor):
    client = stub(nickname='helga')
    stfu.stfu(client, '#bots', 'me', 'helga stfu for 30', 'stfu', ['for', '30'])
    reactor.callLater.assertCalledWith(30*60, stfu.auto_unsilence, client, '#bots', 30*60)


def test_stfu_unsilences_channel():
    stfu.silenced = set(['#bots'])
    resp = stfu.stfu(stub(nickname='helga'), '#bots', 'me', 'helga speak', 'speak', [])
    assert resp in map(lambda x: x.format(nick='me'), stfu.unsilence_acks)
    assert '#bots' not in stfu.silenced



import re

from helga.plugins import match


@match(r'^s/(.*?)/(.*?)/?$')
def meant_to_say(client, channel, nick, message, matches):
    """
    A plugin so users can correct what they have said. For example::

        <sduncan> this is a foo message
        <sduncan> s/foo/bar
        <helga> sduncan meant to say: this is a bar message
    """
    try:
        last = client.last_message[channel][nick]
    except KeyError:
        return None

    old, new = matches[0]
    modified = re.sub(old, new, last)

    # Don't respond if we don't replace anything ... it's annoying
    if modified != last:
        return '{0} meant to say: {1}'.format(nick, modified)



from mock import patch
from pretend import stub

from helga.plugins import facts


def test_term_regex():
    pat = facts.term_regex('foo')

    assert bool(pat.match('foo'))
    assert not bool(pat.match('will not match foo'))


@patch('helga.plugins.facts.db')
def test_show_fact_without_author(db):
    fact = {
        'term': 'foo',
        'fact': 'foo is bar',
    }

    db.facts.find_one.return_value = fact
    assert facts.show_fact('foo') == 'foo is bar'


@patch('helga.plugins.facts.db')
def test_show_fact_without_set_time(db):
    fact = {
        'term': 'foo',
        'fact': 'foo is bar',
        'set_by': 'sduncan',
    }

    db.facts.find_one.return_value = fact
    assert facts.show_fact('foo') == 'foo is bar (sduncan)'


@patch('helga.plugins.facts.db')
def test_show_fact(db):
    fact = {
        'term': 'foo',
        'fact': 'foo is bar',
        'set_by': 'sduncan',
        'set_date': 1360849874.686594,  # 02/14/2013 08:51AM
    }

    db.facts.find_one.return_value = fact
    assert facts.show_fact('foo') == 'foo is bar (sduncan on 02/14/2013 08:51AM)'


@patch('helga.plugins.facts.settings')
@patch('helga.plugins.facts.add_fact')
def test_facts_match_with_nick(add_fact, settings):
    settings.FACTS_REQUIRE_NICKNAME = True
    client = stub(nickname='helga')

    # The format the regex will use
    found = [('helga: foo bar', 'is', '', 'this is the response')]
    add_fact.return_value = 'ok'

    assert 'ok' == facts.facts_match(client, '', '', '', found)
    add_fact.assertCalledWith('foo bar', 'foo bar is this is the response')


@patch('helga.plugins.facts.settings')
@patch('helga.plugins.facts.add_fact')
def test_facts_match_requires_nick(add_fact, settings):
    settings.FACTS_REQUIRE_NICKNAME = True
    client = stub(nickname='helga')

    # The format the regex will use
    found = [('foo bar', 'is', '', 'this is the response')]
    add_fact.return_value = 'ok'

    assert facts.facts_match(client, '', '', '', found) is None


@patch('helga.plugins.facts.add_fact')
def test_facts_as_reply(add_fact):
    # The format the regex will use
    found = [('foo bar', 'is', '<reply>', 'this is the response')]
    add_fact.return_value = 'ok'

    assert 'ok' == facts.facts_match('', '', '', '', found)
    add_fact.assertCalledWith('foo bar', 'this is the response')


@patch('helga.plugins.facts.add_fact')
def test_facts(add_fact):
    # The format the regex will use
    found = [('foo bar', 'is', '', 'this is the response')]
    add_fact.return_value = 'ok'

    assert 'ok' == facts.facts_match('', '', '', '', found)
    add_fact.assertCalledWith('foo bar', 'foo bar is this is the response')



from mock import Mock, patch

from helga.plugins import poems


@patch('helga.plugins.poems.get_random_line')
def test_fix_repitition_replaces(get_random_line):
    poem = ['foo', 'bar', 'foo']
    get_random_line.return_value = 'baz'
    poem = poems.fix_repitition(poem)

    assert poem == ['foo', 'bar', 'baz']


@patch('helga.plugins.poems.get_random_line')
def test_fix_repititions_gives_up_after_retry(get_random_line):
    poem = ['foo', 'bar', 'foo']
    get_random_line.return_value = 'foo'
    poem = poems.fix_repitition(poem)

    assert poem == ['foo', 'bar', 'foo']
    assert poems.get_random_line.call_count == 2


@patch('helga.plugins.poems.get_random_line')
def test_fix_repitition_does_not_replace(get_random_line):
    poem = ['foo', 'bar', 'baz']
    poem = poems.fix_repitition(poem)

    assert poem == ['foo', 'bar', 'baz']
    assert not poems.get_random_line.called


@patch('helga.plugins.poems.db')
def test_add(db):
    poems.add(5, 'foobar')
    assert db.haiku.insert.called


@patch('helga.plugins.poems.db')
def test_remove(db):
    poems.remove(5, 'foobar')
    assert db.haiku.remove.called


@patch('helga.plugins.poems.db')
def test_get_random_line(db):
    result = Mock()
    result.sort = result
    result.count.return_value = 1
    result.limit.return_value = result
    result.skip.return_value = result
    result.next.return_value = {'message': 'fives1'}

    fake_find = Mock(return_value=result)

    db.haiku.find = fake_find
    line = poems.get_random_line(5)
    assert line == 'fives1'


@patch('helga.plugins.poems.db')
def test_get_random_line_returns_none(db):
    db.haiku.find.return_value = db
    db.count.return_value = 0
    assert poems.get_random_line(5) is None


@patch('helga.plugins.poems.add')
@patch('helga.plugins.poems.make_poem')
def test_use_fives(make_poem, add):
    make_poem.return_value = ['one', 'two', 'three']
    poem = poems.use(5, 'foo')
    assert 'foo' in (poem[0], poem[2])


@patch('helga.plugins.poems.add')
@patch('helga.plugins.poems.make_poem')
def test_use_fives_tanka(make_poem, add):
    make_poem.return_value = ['one', 'two', 'three', 'four', 'five']
    poem = poems.use(5, 'foo', poem_type='tanka')
    assert 'foo' in (poem[0], poem[2])


@patch('helga.plugins.poems.add')
@patch('helga.plugins.poems.make_poem')
def test_use_fives_does_not_duplicate(make_poem, add):
    make_poem.return_value = ['foo', 'two', 'three']
    poem = poems.use(5, 'foo')
    assert poem[0] == 'foo'
    assert poem[2] != 'foo'


@patch('helga.plugins.poems.add')
@patch('helga.plugins.poems.make_poem')
def test_use_sevens(make_poem, add):
    make_poem.return_value = ['one', 'two', 'three']
    poem = poems.use(7, 'foo')
    assert poem[1] == 'foo'


@patch('helga.plugins.poems.add')
@patch('helga.plugins.poems.make_poem')
def test_use_sevens_tanka(make_poem, add):
    make_poem.return_value = ['one', 'two', 'three', 'four', 'five']
    poem = poems.use(7, 'foo', poem_type='tanka')
    assert 'foo' in (poem[1], poem[3], poem[4])


@patch('helga.plugins.poems.get_random_line')
def test_make_poem(get_random_line):
    get_random_line.side_effect = ['one', 'two', 'three', 'four', 'five']
    poem = poems.make_poem()
    assert poem == ['one', 'two', 'three']


@patch('helga.plugins.poems.get_random_line')
def test_make_poem_tanka(get_random_line):
    get_random_line.side_effect = ['one', 'two', 'three', 'four', 'five']
    poem = poems.make_poem(poem_type='tanka')
    assert poem == ['one', 'two', 'three', 'four', 'five']



"""
Tests for helga.plugins
"""
from unittest import TestCase

from mock import Mock, patch
from pretend import stub

from helga import plugins


class RegistryTestCase(TestCase):

    def test_prioritized(self):
        fake_plugin = stub(name='foo', priority=50)
        fake_decorated = stub(_plugins=[
            stub(name='bar', priority=10),
            stub(name='baz', priority=0),
            stub(name='qux', priority=99),
        ])
        plugins.registry.plugins = {'foo': fake_plugin, 'bar': fake_decorated}
        plugins.registry.enabled_plugins['#bots'] = set(['foo', 'bar'])

        items = plugins.registry.prioritized('#bots')

        assert items[0].name == 'qux'
        assert items[1].name == 'foo'
        assert items[2].name == 'bar'
        assert items[3].name == 'baz'

    def test_prioritized_reversed(self):
        fake_plugin = stub(name='foo', priority=50)
        fake_decorated = stub(_plugins=[
            stub(name='bar', priority=10),
            stub(name='baz', priority=0),
            stub(name='qux', priority=99),
        ])
        plugins.registry.plugins = {'foo': fake_plugin, 'bar': fake_decorated}
        plugins.registry.enabled_plugins['#bots'] = set(['foo', 'bar'])

        items = plugins.registry.prioritized('#bots', high_to_low=False)

        assert items[3].name == 'qux'
        assert items[2].name == 'foo'
        assert items[1].name == 'bar'
        assert items[0].name == 'baz'

    def test_process_stops_when_async(self):
        things = [Mock(), Mock(), Mock()]

        # Make the middle one raise
        things[0].process.return_value = None
        things[1].process.side_effect = plugins.ResponseNotReady
        things[2].process.return_value = None

        assert [] == plugins.registry.process(None, '#bots', 'me', 'foobar')
        assert not things[2].process.called


class PluginTestCase(TestCase):

    def setUp(self):
        self.plugin = plugins.Plugin()
        self.client = Mock(nickname='helga')

    def test_preprocessor_decorator(self):
        @plugins.preprocessor
        def foo(client, channel, nick, message):
            return 'foo', 'bar', 'baz'

        expected = ('foo', 'bar', 'baz')
        args = (self.client, '#bots', 'me', 'foobar')

        assert hasattr(foo, '_plugins')
        assert len(foo._plugins) == 1
        assert expected == foo(*args)
        assert expected == foo._plugins[0].preprocess(*args)


class CommandTestCase(TestCase):

    def setUp(self):
        self.cmd = plugins.Command('foo', aliases=('bar', 'baz'), help='foo cmd')
        self.client = Mock(nickname='helga')

    def test_init_does_not_overwrite_things(self):
        class MyCommand(plugins.Command):
            command = 'dothis'
            aliases = ('foo', 'bar', 'baz')
            help = 'my command'

        cmd = MyCommand()
        assert cmd.command == 'dothis'
        assert cmd.aliases == ('foo', 'bar', 'baz')
        assert cmd.help == 'my command'

    def test_parse_handles_main_command(self):
        assert 'foo' == self.cmd.parse('helga', 'helga foo')[0]

    @patch('helga.plugins.core.settings')
    def test_parse_handles_char_prefix(self, settings):
        settings.COMMAND_PREFIX_CHAR = '#'
        assert 'foo' == self.cmd.parse('helga', '#foo')[0]

    def test_parse_handles_aliases(self):
        assert 'bar' == self.cmd.parse('helga', 'helga bar')[0]
        assert 'baz' == self.cmd.parse('helga', 'helga baz')[0]

    def test_parse_with_punctuation(self):
        assert 'foo' == self.cmd.parse('helga', 'helga: foo')[0]
        assert 'foo' == self.cmd.parse('helga', 'helga, foo')[0]
        assert 'foo' == self.cmd.parse('helga', 'helga ----> foo')[0]

    def test_parse_does_not_handle(self):
        assert '' == self.cmd.parse('helga', 'helga qux')[0]

    def test_parse_returns_args(self):
        assert ['1', '2', '3'] == self.cmd.parse('helga', 'helga foo 1 2 3')[1]

    def test_process_for_different_command_returns_none(self):
        assert self.cmd.process(self.client, '#bots', 'me', 'helga qux') is None

    def test_process_calls_class_run_method(self):
        self.cmd.run = lambda client, chan, nick, msg, cmd, args: 'run'
        assert 'run' == self.cmd.process(self.client, '#bots', 'me', 'helga foo')

    def test_process_calls_custom_runner(self):
        cmd = plugins.Command('foo', aliases=('bar', 'baz'), help='foo command')
        cmd.run = lambda client, chan, nick, msg, cmd, args: 'runner'
        assert 'runner' == cmd.process(self.client, '#bots', 'me', 'helga foo')

    def test_multiple_decorators(self):
        @plugins.command('foo')
        @plugins.command('bar')
        def foobar(*args):
            return args[-2]

        assert len(foobar._plugins) == 2
        assert 'bar' == foobar._plugins[0](self.client, '#bots', 'me', 'helga bar')
        assert 'foo' == foobar._plugins[1](self.client, '#bots', 'me', 'helga foo')

    def test_decorator_using_command(self):
        @plugins.command('foo')
        def foo(client, chan, nick, msg, cmd, args):
            return 'bar'

        assert 'bar' == foo._plugins[0](self.client, '#bots', 'me', 'helga foo')

    def test_decorator_using_alias(self):
        @plugins.command('foo', aliases=['baz'])
        def foo(client, chan, nick, msg, cmd, args):
            return 'bar'

        assert 'bar' == foo._plugins[0](self.client, '#bots', 'me', 'helga baz')


class MatchTestCase(TestCase):

    def setUp(self):
        self.match = plugins.Match('foo')
        self.client = Mock()

    def test_init_does_not_overwrite_things(self):
        class MyMatch(plugins.Match):
            pattern = 'foo'

        match = MyMatch()
        assert match.pattern == 'foo'

    def test_match_using_callable(self):
        self.match.pattern = lambda m: 'foobar'
        assert 'foobar' == self.match.match('this is a foo message')

    def test_match_using_simple_pattern(self):
        self.match.pattern = r'foo-(\d+)'
        assert ['123'] == self.match.match('this is about foo-123')

    def test_match_returns_none_on_typeerror(self):
        self.match.pattern = Mock(side_effect=TypeError)
        assert self.match.match('this is a foo message') is None

    def test_simple_decorator(self):
        @plugins.match('foo-(\d+)')
        def foo(client, chan, nick, msg, matches):
            return matches[0]

        assert '123' == foo._plugins[0](self.client, '#bots', 'me', 'this is about foo-123')

    def test_callable_decorator(self):
        @plugins.match(lambda x: x.startswith('foo'))
        def foo(client, chan, nick, msg, matches):
            return 'bar'

        assert 'bar' == foo._plugins[0](self.client, '#bots', 'me', 'foo at the start')
        assert foo._plugins[0](self.client, '#bots', 'me', 'not at the start foo') is None



from mock import Mock, patch

from helga.plugins import webhooks


@patch('helga.plugins.webhooks.registry')
def test_route(reg):
    reg.get_plugin.return_value = reg
    fake_fn = lambda: 'foo'
    webhooks.route('/foo', methods=['GET', 'POST'])(fake_fn)

    reg.add_route.assertCalledWith(fake_fn, '/foo', ['GET', 'POST'])


@patch('helga.plugins.webhooks.registry')
def test_route_with_no_methods(reg):
    reg.get_plugin.return_value = reg
    fake_fn = lambda: 'foo'
    webhooks.route('/foo')(fake_fn)

    reg.add_route.assertCalledWith(fake_fn, '/foo', ['GET'])


@patch('helga.plugins.webhooks.settings')
def test_authenticated_passes(settings):
    @webhooks.authenticated
    def fake_fn(*args, **kwargs):
        return 'OK'

    settings.WEBHOOKS_CREDENTIALS = [('foo', 'bar')]

    request = Mock()
    request.getUser.return_value = 'foo'
    request.getPassword.return_value = 'bar'

    assert fake_fn(request) == 'OK'


@patch('helga.plugins.webhooks.settings')
def test_authenticated_fails_when_called(settings):
    @webhooks.authenticated
    def fake_fn(*args, **kwargs):
        return 'OK'

    settings.WEBHOOKS_CREDENTIALS = [('person', 'password')]

    request = Mock()
    request.getUser.return_value = 'foo'
    request.getPassword.return_value = 'bar'

    fake_fn(request)

    request.setRepsonseCode.assertCalledWith(401)



SERVER = {
    'HOST': '66.6.152.52',
    'PORT': 6697,
    'SSL': True,
    'USERNAME': 'helgabot',
    'PASSWORD': 'b!gL4nd',
}

CHANNELS = [('#bottest',)]
NICK = 'helgabot-dev'

WEBHOOKS_CREDENTIALS = [
]

NEWRELIC_WEBHOOK_ANNOUNCE_CHANNEL = '#bottest'
NEWRELIC_API_KEY = '89c546ce2d34d281ed164f8c645d2a0d2c25be7dea1d00f'
NEWRELIC_ACCOUNT_ID = '49454'



from mock import patch
from pretend import stub

from helga.plugins import jira


settings_stub = stub(JIRA_URL='http://example.com/{ticket}',
                     JIRA_SHOW_FULL_DESCRIPTION=False)


@patch('helga.plugins.jira.settings', settings_stub)
@patch('helga.plugins.jira.db')
def test_init_jira_patterns(mock_db):
    mock_db.jira.find.return_value = [{'re': 'foo'}]
    jira.JIRA_PATTERNS.clear()

    assert 'foo' not in jira.JIRA_PATTERNS
    jira.init_jira_patterns()
    assert 'foo' in jira.JIRA_PATTERNS


def test_find_jira_numbers_ignores_url():
    message = 'this has a url http://example.com/foobar-123'
    jira.JIRA_PATTERNS = set('foobar')
    assert jira.find_jira_numbers(message) == []


def test_find_jira_numbers_finds_all():
    message = 'this ia about foo-123, bar-456, baz-789, and qux-000'
    jira.JIRA_PATTERNS = set(['foo', 'bar', 'baz'])

    tickets = jira.find_jira_numbers(message)
    assert 'foo-123' in tickets
    assert 'bar-456' in tickets
    assert 'baz-789' in tickets


def test_find_jira_numbers_ignores_unknown():
    message = 'this ia about foo-123, bar-456, baz-789, and qux-000'
    jira.JIRA_PATTERNS = set(['foo', 'bar', 'baz'])

    tickets = jira.find_jira_numbers(message)
    assert 'quz-000' not in tickets


@patch('helga.plugins.jira.db')
def test_add_re_inserts_new_record(db):
    db.jira.find.return_value = db
    db.count.return_value = 0
    jira.JIRA_PATTERNS.clear()

    jira.add_re('foo')

    assert 'foo' in jira.JIRA_PATTERNS
    assert db.jira.insert.called


@patch('helga.plugins.jira.db')
def test_add_re_has_existing_record_in_db(db):
    db.jira.find.return_value = db
    db.count.return_value = 1
    jira.JIRA_PATTERNS.clear()

    jira.add_re('foo')

    assert 'foo' in jira.JIRA_PATTERNS
    assert not db.jira.insert.called


@patch('helga.plugins.jira.db')
def test_remove_re_does_removing(db):
    jira.remove_re('foo')
    assert db.jira.remove.called


@patch('helga.plugins.jira.db')
def test_remove_re_removes_ticket(db):
    jira.JIRA_PATTERNS = set(['foo'])
    jira.remove_re('foo')

    assert db.jira.remove.called
    assert 'foo' not in jira.JIRA_PATTERNS


@patch('helga.plugins.jira.settings', settings_stub)
def test_jira_match():
    expected = 'me might be talking about JIRA ticket: http://example.com/foo-123'
    assert expected == jira.jira_match(None, '#bots', 'me', 'this is about foo-123', ['foo-123'])


@patch('helga.plugins.jira.settings', settings_stub)
def test_jira_match_multiple():
    resp = jira.jira_match(None, '#bots', 'me', 'foo-123 and bar-456', ['foo-123', 'bar-456'])
    assert 'http://example.com/foo-123' in resp
    assert 'http://example.com/bar-456' in resp



from mock import patch, Mock

from helga.plugins import manager


@patch('helga.plugins.manager.db')
@patch('helga.plugins.manager.registry')
def test_auto_enable_plugins(plugins, db):
    client = Mock()
    rec = {'plugin': 'haiku', 'channels': ['a', 'b', 'c']}
    db.auto_enabled_plugins.find.return_value = [rec]

    manager.auto_enable_plugins(client)
    plugins.enable.assertCalledWith('a', 'haiku')
    plugins.enable.assertCalledWith('b', 'haiku')
    plugins.enable.assertCalledWith('c', 'haiku')


@patch('helga.plugins.manager.registry')
def test_list_plugins(plugins):
    client = Mock()
    plugins.plugins = {
        'plugin1': [1, 2, 3],
        'plugin2': [4, 5, 6],
        'plugin3': [7, 8, 9],
    }
    plugins.all_plugins = set(plugins.plugins.keys())
    plugins.enabled_plugins = {'foo': set(['plugin2'])}

    resp = manager.list_plugins(client, 'foo')
    assert 'Plugins enabled on this channel: plugin2' in resp
    assert 'Available plugins: plugin1, plugin3' in resp


@patch('helga.plugins.manager.db')
@patch('helga.plugins.manager.registry')
def test_enable_plugins_inits_record(plugins, db):
    client = Mock()

    plugins.all_plugins = ['foobar']

    db.auto_enabled_plugins.find_one.return_value = None
    manager.enable_plugins(client, '#bots', 'foobar')

    assert db.auto_enabled_plugins.insert.called


@patch('helga.plugins.manager.db')
@patch('helga.plugins.manager.registry')
def test_enable_plugins_updates_record(plugins, db):
    client = Mock()

    plugins.all_plugins = ['foobar']

    rec = {'plugin': 'foobar', 'channels': ['#all']}
    db.auto_enabled_plugins.find_one.return_value = rec
    manager.enable_plugins(client, '#bots', 'foobar')

    assert db.auto_enabled_plugins.save.called
    assert '#bots' in rec['channels']


@patch('helga.plugins.manager.db')
@patch('helga.plugins.manager.registry')
def test_disable_plugins(plugins, db):
    client = Mock()
    plugins.all_plugins = ['foobar']

    rec = {'plugin': 'foobar', 'channels': ['#all', '#bots']}
    db.auto_enabled_plugins.find_one.return_value = rec
    manager.disable_plugins(client, '#bots', 'foobar')

    assert db.auto_enabled_plugins.save.called
    assert '#bots' not in rec['channels']



import re

from collections import defaultdict
from pretend import stub

from helga.plugins import meant_to_say


def test_meant_to_say_none_when_no_last_message():
    client = stub(last_message=defaultdict(dict))
    assert meant_to_say.meant_to_say(client, '#bots', 'me', 's/foo/bar', [('foo', 'bar')]) is None


def test_meant_to_say_regex():
    plugin = meant_to_say.meant_to_say._plugins[0]
    assert bool(re.match(plugin.pattern, 's/foo/bar'))
    assert bool(re.match(plugin.pattern, 's/foo/bar/'))


def test_meant_to_say_returns_modified():
    client = stub(last_message=defaultdict(dict))
    client.last_message['#bots']['me'] = 'this is a foo message'

    resp = meant_to_say.meant_to_say(client, '#bots', 'me', 's/foo/bar', [('foo', 'bar')])
    assert resp == 'me meant to say: this is a bar message'


def test_meant_to_say_none_when_not_modified():
    client = stub(last_message=defaultdict(dict))
    client.last_message['#bots']['me'] = 'this is a message'

    assert meant_to_say.meant_to_say(client, '#bots', 'me', 's/foo/bar', [('foo', 'bar')]) is None



from mock import Mock, patch
from pretend import stub

from helga.plugins import operator, ACKS


def test_operator_ignores_non_oper_user():
    client = stub(operators=['me'])
    assert operator.operator(client, '#bots', 'sduncan', 'do something', '', '') in operator.nopes


def test_operator_join_calls_client_join():
    client = Mock(operators=['me'])
    operator.operator(client, '#bots', 'me', 'do something', 'op', ['join', '#foo'])
    client.join.assertCalledWith('#foo')


def test_operator_join_ignores_invalid_channel():
    client = Mock(operators=['me'])
    operator.operator(client, '#bots', 'me', 'do something', 'op', ['join', 'foo'])
    assert not client.join.called


def test_operator_leave_calls_client_leave():
    client = Mock(operators=['me'])
    operator.operator(client, '#bots', 'me', 'do something', 'op', ['leave', '#foo'])
    client.leave.assertCalledWith('#foo')


def test_operator_leave_ignores_invalid_channel():
    client = Mock(operators=['me'])
    operator.operator(client, '#bots', 'me', 'do something', 'op', ['leave', 'foo'])
    assert not client.leave.called


@patch('helga.plugins.operator.db')
def test_add_autojoin_exists(db):
    db.autojoin.find.return_value = db
    db.count.return_value = 1
    assert operator.add_autojoin('#foo') not in ACKS


@patch('helga.plugins.operator.db')
def test_add_autojoin_adds(db):
    db.autojoin.find.return_value = db
    db.count.return_value = 0
    operator.add_autojoin('foo')
    db.autojoin.insert.assertCalledWith({'channel': 'foo'})


@patch('helga.plugins.operator.db')
def test_remove_autojoin(db):
    operator.remove_autojoin('foo')
    db.autojoin.remove.assertCalledWith({'channel': 'foo'})



# -*- coding: utf8 -*-
import re

from mock import Mock, patch
from unittest import TestCase

from helga import comm


class ClientTestCase(TestCase):

    def setUp(self):
        self.client = comm.Client()

    def test_parse_nick(self):
        nick = self.client.parse_nick('foo!~foobar@localhost')
        assert nick == 'foo'

    @patch('helga.comm.irc.IRCClient')
    def test_msg_encodes_utf_8(self, irc):
        uni_str = u'ಠ_ಠ'
        utf8_str = '\xe0\xb2\xa0_\xe0\xb2\xa0'

        self.client.msg('#foo', uni_str)
        irc.msg.assertCalledWith('#foo', utf8_str)

    def test_alterCollidedNick(self):
        self.client.alterCollidedNick('foo')
        assert re.match(r'foo_[\d]+', self.client.nickname)

    @patch('helga.comm.settings')
    @patch('helga.comm.smokesignal')
    def test_signedOn_sends_signal(self, signal, settings):
        settings.CHANNELS = []
        self.client.signedOn()
        signal.emit.assertCalledWith('signon')

    @patch('helga.comm.registry')
    def test_privmsg_sends_single_string(self, registry):
        self.client.msg = Mock()
        registry.process.return_value = ['line1', 'line2']

        self.client.privmsg('foo!~bar@baz', '#bots', 'this is the input')

        args = self.client.msg.call_args[0]
        assert args[0] == '#bots'
        assert args[1] == 'line1\nline2'

    @patch('helga.comm.registry')
    def test_privmsg_responds_to_user_when_private(self, registry):
        self.client.nickname = 'helga'
        self.client.msg = Mock()
        registry.process.return_value = ['line1', 'line2']

        self.client.privmsg('foo!~bar@baz', 'helga', 'this is the input')

        assert self.client.msg.call_args[0][0] == 'foo'
